package io.provis.maven.provision;

import io.provis.aether.ArtifactResolver;
import io.tesla.proviso.archive.Archiver;

import java.io.File;
import java.io.IOException;

import javax.inject.Inject;
import javax.inject.Named;


@Named
public class MavenProvisioner {

  @Inject
  @Named("${basedir}")
  private File basedir;

  @Inject
  private ArtifactResolver artifactResolver;

  @Inject
  private Archiver archiver;

  public File provision(String mavenVersion) throws IOException {    
    if (mavenVersion == null || mavenVersion.length() <= 0) {
      throw new IllegalArgumentException("Maven version not specified");
    }

    File installDir = new File(basedir, "target/maven");
    File homeDir = new File(installDir, "apache-maven-" + mavenVersion);
    if (homeDir.list() != null && homeDir.list().length > 0) {
      return homeDir;
    }

    File binZip = artifactResolver.resolve("org.apache.maven:apache-maven:zip:bin:" + mavenVersion);

    installDir.mkdirs();
    if (!installDir.isDirectory()) {
      throw new IllegalStateException("Could not create Maven install directory " + installDir);
    }

    archiver.unarchive(binZip, installDir);

    File mvn = new File(homeDir, "bin/mvn");
    if (!mvn.isFile()) {
      throw new IllegalStateException("Unpacking of Maven distro failed");
    }
    mvn.setExecutable(true);

    return homeDir;
  }
}
